from edc_constants.constants import YES

from edc_screening.screening_eligibility import FC, ScreeningEligibility


class MyScreeningEligibility(ScreeningEligibility):
    def __init__(self, **kwargs):
        self.age_in_years = None
        self.alive = None
        super().__init__(**kwargs)

    def get_required_fields(self):
        return {
            "age_in_years": FC(lambda x: x >= 18, "must be >=18"),
            "alive": FC(YES, "must be alive"),
        }



from django.db import models
from edc_constants.constants import YES
from edc_model.models import BaseUuidModel

from edc_screening.model_mixins import EligibilityModelMixin

from ..model_mixins import ScreeningModelMixin
from .eligibility import MyScreeningEligibility


class SubjectScreening(ScreeningModelMixin, BaseUuidModel):

    thing = models.CharField(max_length=10, null=True)


class SubjectScreeningWithEligibility(
    ScreeningModelMixin, EligibilityModelMixin, BaseUuidModel
):

    eligibility_cls = MyScreeningEligibility

    alive = models.CharField(max_length=10, default=YES)


class SubjectScreeningWithEligibilitySimple(
    ScreeningModelMixin, EligibilityModelMixin, BaseUuidModel
):

    pass



import re

from django.core.exceptions import ObjectDoesNotExist
from django.test import TestCase, override_settings
from edc_constants.constants import MALE, NO, UUID_PATTERN, YES
from edc_identifier.models import IdentifierModel

from edc_screening.age_evaluator import AgeEvaluator
from edc_screening.constants import ELIGIBLE, NOT_ELIGIBLE
from edc_screening.gender_evaluator import GenderEvaluator
from edc_screening.utils import (
    eligibility_display_label,
    format_reasons_ineligible,
    get_subject_screening_app_label,
    get_subject_screening_model,
    get_subject_screening_model_cls,
)

from .models import (
    SubjectScreening,
    SubjectScreeningWithEligibility,
    SubjectScreeningWithEligibilitySimple,
)


class TestScreening(TestCase):
    @override_settings(SUBJECT_SCREENING_MODEL="edc_screening.subjectscreening")
    def test_model_funcs(self):
        self.assertEqual(get_subject_screening_model(), "edc_screening.subjectscreening")
        self.assertEqual(get_subject_screening_app_label(), "edc_screening")
        self.assertEqual(get_subject_screening_model_cls(), SubjectScreening)

    def test_format_reasons_ineligible(self):

        str_values = ["age_in_years", "on_art"]
        self.assertEqual(format_reasons_ineligible(*str_values), "age_in_years|on_art")

        str_values = ["age_in_years", None, None, "on_art"]
        self.assertEqual(
            format_reasons_ineligible(*str_values, delimiter="|"), "age_in_years|on_art"
        )

        str_values = ["age_in_years", None, None, "on_art"]
        self.assertEqual(
            format_reasons_ineligible(*str_values, delimiter="<BR>"), "age_in_years<BR>on_art"
        )

        str_values = []
        self.assertEqual(format_reasons_ineligible(*str_values, delimiter="<BR>"), None)

        str_values = None
        self.assertEqual(format_reasons_ineligible(str_values, delimiter="<BR>"), None)

    def test_eligibility_display_label(self):
        self.assertEqual(eligibility_display_label(True), ELIGIBLE.upper())
        self.assertEqual(eligibility_display_label(False), NOT_ELIGIBLE)

    def test_age_evaluator(self):
        age_evaluator = AgeEvaluator(age_lower=18, age_lower_inclusive=True)
        self.assertFalse(age_evaluator.eligible())
        self.assertFalse(age_evaluator.eligible(17))
        self.assertTrue(age_evaluator.eligible(18))

        age_evaluator.eligible()
        self.assertEqual(age_evaluator.reasons_ineligible, "Age unknown")

    def test_gender_evaluator(self):
        gender_evaluator = GenderEvaluator(MALE)
        self.assertTrue(gender_evaluator.eligible)
        gender_evaluator = GenderEvaluator("BARK")
        self.assertFalse(gender_evaluator.eligible)

    def test_model(self):
        obj = SubjectScreening.objects.create(age_in_years=25)

        try:
            IdentifierModel.objects.get(identifier=obj.screening_identifier)
        except ObjectDoesNotExist:
            self.fail(f"Identifier unexpectedly not found. {obj.screening_identifier}")

        self.assertTrue(re.match(UUID_PATTERN, obj.subject_identifier))

        screening_identifier = obj.screening_identifier
        obj.save()
        obj.refresh_from_db()
        self.assertEqual(screening_identifier, obj.screening_identifier)

        self.assertTrue(re.match(UUID_PATTERN, obj.subject_identifier))

        obj.subject_identifier = "1234"
        obj.save()
        obj.refresh_from_db()
        self.assertIsNone(re.match(UUID_PATTERN, obj.subject_identifier))

    def test_model_with_screening_eligiblity_cls_missing(self):
        """No criteria is being assessed"""
        self.assertFalse(
            SubjectScreeningWithEligibilitySimple.objects.create(
                age_in_years=25,
            ).eligible
        )

    def test_model_screening_eligiblity_resave(self):
        obj = SubjectScreeningWithEligibility.objects.create(age_in_years=17, alive=NO)
        self.assertFalse(obj.eligible)
        # note model attr is formatted as a string of dict.values()
        self.assertEqual(obj.reasons_ineligible, "must be >=18|must be alive")
        obj.save()
        obj.refresh_from_db()
        self.assertFalse(obj.eligible)
        self.assertEqual(obj.reasons_ineligible, "must be >=18|must be alive")
        obj.age_in_years = 18
        obj.save()
        obj.refresh_from_db()
        self.assertFalse(obj.eligible)
        self.assertEqual(obj.reasons_ineligible, "must be alive")
        obj.alive = YES
        obj.save()
        obj.refresh_from_db()
        self.assertTrue(obj.eligible)
        self.assertEqual(obj.reasons_ineligible, None)



from __future__ import annotations

from typing import TYPE_CHECKING

from django.apps import apps as django_apps
from django.conf import settings
from django.utils.html import format_html

from .constants import ELIGIBLE, NOT_ELIGIBLE

if TYPE_CHECKING:
    from .model_mixins import ScreeningModelMixin


def get_subject_screening_app_label() -> str:
    return get_subject_screening_model().split(".")[0]


def get_subject_screening_model() -> str:
    return getattr(settings, "SUBJECT_SCREENING_MODEL")


def get_subject_screening_model_cls() -> ScreeningModelMixin:
    return django_apps.get_model(get_subject_screening_model())


def format_reasons_ineligible(*str_values: str, delimiter=None) -> str:
    reasons = None
    delimiter = delimiter or "|"
    str_values = tuple(x for x in str_values if x is not None)
    if str_values:
        reasons = format_html(delimiter.join(str_values))
    return reasons


def eligibility_display_label(eligible) -> str:
    return ELIGIBLE.upper() if eligible else NOT_ELIGIBLE



from edc_auth.auth_objects import (
    AUDITOR_ROLE,
    CLINICIAN_ROLE,
    CLINICIAN_SUPER_ROLE,
    NURSE_ROLE,
)
from edc_auth.site_auths import site_auths
from edc_auth.utils import remove_default_model_permissions_from_edc_permissions

from .auth_objects import SCREENING, SCREENING_ROLE, SCREENING_SUPER, SCREENING_VIEW

site_auths.add_post_update_func(
    "edc_screening", remove_default_model_permissions_from_edc_permissions
)

site_auths.add_custom_permissions_tuples(
    model="edc_screening.edcpermissions",
    codename_tuples=(
        ("edc_screening.view_screening_listboard", "Can access Screening listboard"),
        ("edc_screening.nav_screening_section", "Can access screening section"),
    ),
)

site_auths.add_group(
    "edc_screening.view_screening_listboard",
    "edc_screening.nav_screening_section",
    name=SCREENING,
)

site_auths.add_group(
    "edc_screening.view_screening_listboard",
    "edc_screening.nav_screening_section",
    name=SCREENING_SUPER,
)

site_auths.add_group(
    "edc_screening.view_screening_listboard",
    "edc_screening.nav_screening_section",
    name=SCREENING_VIEW,
)

site_auths.add_role(SCREENING, name=SCREENING_ROLE)
site_auths.update_role(SCREENING, name=CLINICIAN_ROLE)
site_auths.update_role(SCREENING, name=NURSE_ROLE)
site_auths.update_role(SCREENING_SUPER, name=CLINICIAN_SUPER_ROLE)
site_auths.update_role(SCREENING_VIEW, name=AUDITOR_ROLE)



from edc_auth.model_mixins import EdcPermissionsModelMixin
from edc_model.models import BaseUuidModel


class EdcPermissions(EdcPermissionsModelMixin, BaseUuidModel):

    # see edc_auth for permissions attached to this model

    class Meta(EdcPermissionsModelMixin.Meta):
        pass



from django.db import models
from edc_identifier.model_mixins import NonUniqueSubjectIdentifierModelMixin
from edc_search.model_mixins import SearchSlugModelMixin

from ..screening_identifier import ScreeningIdentifier
from ..stubs import SubjectScreeningModelStub


class ScreeningIdentifierModelMixin(
    NonUniqueSubjectIdentifierModelMixin, SearchSlugModelMixin, models.Model
):

    identifier_cls = ScreeningIdentifier
    screening_identifier_field_name: str = "screening_identifier"

    def save(self, *args, **kwargs):
        """Screening Identifier is always allocated."""
        if not self.id:
            setattr(
                self,
                self.screening_identifier_field_name,
                self.identifier_cls().identifier,
            )
        super().save(*args, **kwargs)  # type:ignore

    def update_subject_identifier_on_save(self: SubjectScreeningModelStub) -> str:
        """Overridden to not create a new study-allocated subject
        identifier on save.

        Only gets called if self.id is None (create).

        Instead, just copy subject_identifier_as_pk to
        subject_identifier to maintain uniqueness.

        The "real" subject_identifier will be set upon consent.

        See SubjectIdentifierMethodsModelMixin. \
            update_subject_identifier_on_save()
        """
        if not self.subject_identifier:
            self.subject_identifier = self.make_new_identifier()
        return self.subject_identifier

    def make_new_identifier(self) -> str:
        return self.subject_identifier_as_pk.hex

    class Meta:
        abstract = True



from datetime import datetime
from typing import Any, Protocol
from uuid import UUID


class ModelStub(Protocol):
    def save(self, *args, **kwargs):
        ...


class SubjectScreeningModelStub(ModelStub, Protocol):
    eligibility_cls: Any
    identifier_cls: Any
    screening_identifier_field_name: str

    id: UUID
    screening_identifier: str
    subject_identifier: str
    subject_identifier_as_pk: UUID
    report_datetime: datetime
    gender: str
    age_in_years: int
    eligible: bool
    reasons_ineligible: str
    ...

    def get_report_datetime_for_eligibility_datetime(self) -> datetime:
        ...



from .eligibility_model_mixin import EligibilityModelMixin
from .screening_fields_model_mixin import ScreeningFieldsModeMixin
from .screening_identifier_model_mixin import ScreeningIdentifierModelMixin
from .screening_methods_model_mixin import ScreeningMethodsModeMixin
from .screening_model_mixin import ScreeningModelMixin



from django.db import models

from ..screening_identifier import ScreeningIdentifier
from .screening_fields_model_mixin import ScreeningFieldsModeMixin
from .screening_identifier_model_mixin import ScreeningIdentifierModelMixin
from .screening_methods_model_mixin import ScreeningMethodsModeMixin


class ScreeningModelMixin(
    ScreeningMethodsModeMixin,
    ScreeningIdentifierModelMixin,
    ScreeningFieldsModeMixin,
    models.Model,
):

    """You may wish to also include the `EligibilityModelMixin`
    in your declaration."""

    identifier_cls = ScreeningIdentifier

    class Meta:
        abstract = True



from django.test import TestCase
from edc_constants.constants import NO, TBD, YES

from ..screening_eligibility import FC
from ..screening_eligibility import ScreeningEligibility as BaseScreeningEligibility
from ..screening_eligibility import (
    ScreeningEligibilityAttributeError,
    ScreeningEligibilityError,
    ScreeningEligibilityInvalidCombination,
    ScreeningEligibilityModelAttributeError,
)
from .models import SubjectScreening


class TestScreening(TestCase):
    def test_fc(self):
        fc = FC()
        self.assertTrue(repr(fc))
        self.assertTrue(str(fc))

    def test_repr_and_str(self):
        class ScreeningEligibility(BaseScreeningEligibility):
            pass

        eligibility = ScreeningEligibility()
        self.assertTrue(repr(eligibility))
        self.assertTrue(str(eligibility))

    def test_without_required_fields_or_assess_eligibility(self):
        """Assert that the base screening class, if not modified,
        does nothing.
        """

        class ScreeningEligibility(BaseScreeningEligibility):
            pass

        eligibility = ScreeningEligibility()
        self.assertDictEqual(eligibility.reasons_ineligible, {})
        self.assertFalse(eligibility.is_eligible)
        self.assertEqual(eligibility.eligible, TBD)

    def test_required_fields_does_not_have_corresponding_class_attr(self):
        required_fields = dict(erik=FC(YES, "erik must be YES", ignore_if_missing=True))

        class ScreeningEligibility(BaseScreeningEligibility):
            def get_required_fields(self):
                return required_fields

        cleaned_data = {}
        self.assertRaises(
            ScreeningEligibilityAttributeError,
            ScreeningEligibility,
            cleaned_data=cleaned_data,
        )

    def test_required_fields_has_corresponding_class_attr(self):
        required_fields = dict(erik=FC(YES, "erik must be YES"))

        class ScreeningEligibility(BaseScreeningEligibility):
            def __init__(self, **kwargs):
                self.erik = None
                super().__init__(**kwargs)

            def get_required_fields(self):
                return required_fields

        cleaned_data = dict(erik=YES)
        try:
            ScreeningEligibility(cleaned_data=cleaned_data)
        except ScreeningEligibilityAttributeError:
            self.fail("ScreeningEligibilityAttributeError unexpectedly raised")

    def test_required_fields_does_not_have_corresponding_model_attr(self):
        model_obj = SubjectScreening.objects.create(age_in_years=25)
        required_fields = dict(erik=FC(NO, "erik must be YES"))

        class ScreeningEligibility(BaseScreeningEligibility):
            def __init__(self, **kwargs):
                self.erik = None
                super().__init__(**kwargs)

            def get_required_fields(self):
                return required_fields

        self.assertRaises(
            ScreeningEligibilityModelAttributeError, ScreeningEligibility, model_obj=model_obj
        )

    def test_with_model_obj(self):
        model_obj = SubjectScreening.objects.create(thing="thing", age_in_years=25)
        required_fields = dict(thing=FC(YES, "thing must be YES"))

        class ScreeningEligibility(BaseScreeningEligibility):
            def __init__(self, **kwargs):
                self.thing = None
                super().__init__(**kwargs)

            def get_required_fields(self):
                return required_fields

        ScreeningEligibility(model_obj=model_obj)

    def test_with_cleaned_data(self):
        cleaned_data = dict(erik=NO)
        required_fields = dict(erik=FC(YES, "erik must be YES"))

        class ScreeningEligibility(BaseScreeningEligibility):
            def __init__(self, **kwargs):
                self.erik = None
                super().__init__(**kwargs)

            def get_required_fields(self):
                return required_fields

        ScreeningEligibility(cleaned_data=cleaned_data)

    def test_not_eligible(self):
        cleaned_data = dict(erik=NO)
        required_fields = dict(erik=FC(YES, "erik must be YES"))

        class ScreeningEligibility(BaseScreeningEligibility):
            def __init__(self, **kwargs):
                self.erik = None
                super().__init__(**kwargs)

            def get_required_fields(self):
                return required_fields

        eligibility = ScreeningEligibility(cleaned_data=cleaned_data)
        self.assertIn("erik", eligibility.reasons_ineligible)
        self.assertEqual(NO, eligibility.eligible)
        self.assertFalse(eligibility.is_eligible)

    def test_eligible(self):
        cleaned_data = dict(erik=YES)
        required_fields = dict(erik=FC(YES, "erik must be YES"))

        class ScreeningEligibility(BaseScreeningEligibility):
            def __init__(self, **kwargs):
                self.erik = None
                super().__init__(**kwargs)

            def get_required_fields(self):
                return required_fields

        eligibility = ScreeningEligibility(cleaned_data=cleaned_data)
        self.assertNotIn("erik", eligibility.reasons_ineligible)
        self.assertEqual(YES, eligibility.eligible)
        self.assertTrue(eligibility.is_eligible)

    def test_custom_assessment(self):
        cleaned_data = dict(erik=YES, thing_one="maybe")
        required_fields = dict(erik=FC(YES, "erik must be YES"))

        class ScreeningEligibility(BaseScreeningEligibility):
            def __init__(self, **kwargs):
                self.erik = None
                super().__init__(**kwargs)

            def get_required_fields(self):
                return required_fields

            def assess_eligibility(self) -> None:
                if self.cleaned_data.get("thing_one") == "maybe" and self.eligible:
                    self.eligible = TBD
                    self.reasons_ineligible.update(thing_one="thing one not sure")

        eligibility = ScreeningEligibility(cleaned_data=cleaned_data)
        self.assertIn("thing_one", eligibility.reasons_ineligible)
        self.assertEqual(TBD, eligibility.eligible)
        self.assertFalse(eligibility.is_eligible)

    def test_invalid_value_for_eligible(self):
        cleaned_data = dict(erik=YES, thing_one="maybe")
        required_fields = dict(erik=FC(YES, "erik must be YES"))

        class ScreeningEligibility(BaseScreeningEligibility):
            def __init__(self, **kwargs):
                self.erik = None
                super().__init__(**kwargs)

            def get_required_fields(self):
                return required_fields

            def assess_eligibility(self) -> None:
                if self.cleaned_data.get("thing_one") == "maybe" and self.eligible == YES:
                    self.eligible = "BAD DOG"
                    self.reasons_ineligible.update(thing_one="thing one not sure")

        self.assertRaises(
            ScreeningEligibilityError, ScreeningEligibility, cleaned_data=cleaned_data
        )

    def test_invalid_combination_eligible_and_reasons(self):
        cleaned_data = dict(erik=YES, thing_one="maybe")
        required_fields = dict(erik=FC(YES, "erik must be YES"))

        class ScreeningEligibility(BaseScreeningEligibility):
            def __init__(self, **kwargs):
                self.erik = None
                super().__init__(**kwargs)

            def get_required_fields(self):
                return required_fields

            def assess_eligibility(self) -> None:
                if self.cleaned_data.get("thing_one") == "maybe":
                    self.eligible = YES
                    self.reasons_ineligible.update(thing_one="thing one not sure")

        self.assertRaises(
            ScreeningEligibilityInvalidCombination,
            ScreeningEligibility,
            cleaned_data=cleaned_data,
        )

    def test_cannot_override_if_ineligible(self):
        """Assert does not call `assess_eligibility` if default assessment
        is ineligible
        """
        cleaned_data = dict(erik=NO, thing_one="maybe")
        required_fields = dict(erik=FC(YES, "erik must be YES"))

        class ScreeningEligibility(BaseScreeningEligibility):
            def __init__(self, **kwargs):
                self.erik = None
                super().__init__(**kwargs)

            def get_required_fields(self):
                return required_fields

            def assess_eligibility(self) -> None:
                self.eligible = YES
                self.reasons_ineligible = {}

        eligibility = ScreeningEligibility(cleaned_data=cleaned_data)
        self.assertIn("erik", eligibility.reasons_ineligible)
        self.assertEqual(NO, eligibility.eligible)
        self.assertFalse(eligibility.is_eligible)

    def test_options(self):
        required_fields = dict(erik=FC(YES, "erik must be YES"))
        options = {
            "eligible_value_default": "PENDING",
            "eligible_values_list": ["YEAH", "NOPE", "PENDING"],
            "is_eligible_value": "YEAH",
            "is_ineligible_value": "NOPE",
        }

        class ScreeningEligibility(BaseScreeningEligibility):
            def __init__(self, **kwargs):
                self.erik = None
                super().__init__(**kwargs)

            def get_required_fields(self):
                return required_fields

        cleaned_data = dict(erik=YES)
        eligibility = ScreeningEligibility(cleaned_data=cleaned_data, **options)
        self.assertEqual(eligibility.eligible, "YEAH")

        cleaned_data = dict(erik=NO)
        eligibility = ScreeningEligibility(cleaned_data=cleaned_data, **options)
        self.assertEqual(eligibility.eligible, "NOPE")

    def test_formatted(self):
        required_fields = dict(
            thing_one=FC(YES, "thing one must be YES"),
            thing_two=FC(YES, "thing two must be YES"),
        )

        class ScreeningEligibility(BaseScreeningEligibility):
            def __init__(self, **kwargs):
                self.thing_one = None
                self.thing_two = None
                super().__init__(**kwargs)

            def get_required_fields(self):
                return required_fields

        cleaned_data = dict(thing_one=NO, thing_two=NO)
        eligibility = ScreeningEligibility(cleaned_data=cleaned_data)
        self.assertEqual(
            eligibility.formatted_reasons_ineligible(),
            "thing one must be YES<BR>thing two must be YES",
        )

        cleaned_data = dict(thing_one=YES, thing_two=YES)
        eligibility = ScreeningEligibility(cleaned_data=cleaned_data)
        self.assertEqual(eligibility.formatted_reasons_ineligible(), "")

    def test_fc_with_callable(self):
        required_fields = dict(
            age_in_years=FC(lambda x: x >= 18, "must be >=18y"),
        )

        class ScreeningEligibility(BaseScreeningEligibility):
            def __init__(self, **kwargs):
                self.age_in_years = None
                super().__init__(**kwargs)

            def get_required_fields(self):
                return required_fields

        cleaned_data = dict(age_in_years=17)
        eligibility = ScreeningEligibility(cleaned_data=cleaned_data)
        self.assertFalse(eligibility.is_eligible)

        cleaned_data = dict(age_in_years=18)
        eligibility = ScreeningEligibility(cleaned_data=cleaned_data)
        self.assertTrue(eligibility.is_eligible)

    def test_eligibility_display_label(self):

        required_fields = dict(
            thing_one=FC(YES, "thing one must be YES"),
        )

        class ScreeningEligibility(BaseScreeningEligibility):
            def __init__(self, **kwargs):
                self.thing_one = None
                super().__init__(**kwargs)

            def get_required_fields(self):
                return required_fields

        cleaned_data = dict(thing_one=NO)
        eligibility = ScreeningEligibility(
            cleaned_data=cleaned_data, eligible_display_label="E", ineligible_display_label="I"
        )
        self.assertEqual(eligibility.display_label, "I")

        cleaned_data = dict(thing_one=YES)
        eligibility = ScreeningEligibility(
            cleaned_data=cleaned_data, eligible_display_label="E", ineligible_display_label="I"
        )
        self.assertEqual(eligibility.display_label, "E")

    def test_missing_data(self):

        required_fields = dict(
            thing_one=FC(YES, "thing one must be YES"),
            thing_two=FC(ignore_if_missing=True),
            thing_three=FC(YES, "thing two must be YES"),
        )

        class ScreeningEligibility(BaseScreeningEligibility):
            def __init__(self, **kwargs):
                self.thing_one = None
                self.thing_two = None
                self.thing_three = None
                super().__init__(**kwargs)

            def get_required_fields(self):
                return required_fields

        cleaned_data = dict(
            thing_one=YES,
            thing_two=None,
            thing_three=None,
        )
        eligibility = ScreeningEligibility(
            cleaned_data=cleaned_data, eligible_display_label="E", ineligible_display_label="I"
        )

        self.assertNotIn("thing_one", eligibility.reasons_ineligible)
        self.assertNotIn("thing_two", eligibility.reasons_ineligible)
        self.assertIn("thing_three", eligibility.reasons_ineligible)
        self.assertEqual(eligibility.display_label, eligibility.eligible_value_default)

        # assert has enough to know ineligible (thing_one=NO)
        cleaned_data = dict(
            thing_one=NO,
            thing_two=None,
            thing_three=None,
        )
        eligibility = ScreeningEligibility(
            cleaned_data=cleaned_data, eligible_display_label="E", ineligible_display_label="I"
        )

        self.assertIn("thing_one", eligibility.reasons_ineligible)
        self.assertNotIn("thing_two", eligibility.reasons_ineligible)
        self.assertIn("thing_three", eligibility.reasons_ineligible)
        self.assertEqual(eligibility.eligible, eligibility.is_ineligible_value)

    def test_missing_data_with_custom_missing_value(self):

        required_fields = dict(
            thing_one=FC(YES, "thing one must be YES"),
            thing_two=FC(YES, missing_value="NOT_ANSWERED"),
        )

        class ScreeningEligibility(BaseScreeningEligibility):
            def __init__(self, **kwargs):
                self.thing_one = None
                self.thing_two = None
                super().__init__(**kwargs)

            def get_required_fields(self):
                return required_fields

        cleaned_data = dict(
            thing_one=YES,
            thing_two=YES,
        )
        eligibility = ScreeningEligibility(
            cleaned_data=cleaned_data, eligible_display_label="E", ineligible_display_label="I"
        )

        self.assertNotIn("thing_one", eligibility.reasons_ineligible)
        self.assertNotIn("thing_two", eligibility.reasons_ineligible)

        required_fields = dict(
            thing_one=FC(YES, "thing one must be YES"),
            thing_two=FC("NOT_ANSWERED", missing_value="NOT_ANSWERED"),
        )

        cleaned_data = dict(
            thing_one=YES,
            thing_two="NOT_ANSWERED",
        )
        eligibility = ScreeningEligibility(cleaned_data=cleaned_data)

        self.assertNotIn("thing_one", eligibility.reasons_ineligible)
        self.assertIn("thing_two", eligibility.reasons_ineligible)
        self.assertIn("not answered", eligibility.reasons_ineligible.get("thing_two"))

        cleaned_data = dict(
            thing_one=YES,
            thing_two=None,
        )
        eligibility = ScreeningEligibility(cleaned_data=cleaned_data)

        self.assertNotIn("thing_one", eligibility.reasons_ineligible)
        self.assertIn("thing_two", eligibility.reasons_ineligible)
        self.assertIn("not answered", eligibility.reasons_ineligible.get("thing_two"))

    def test_invalid_combinations(self):
        class ScreeningEligibility(BaseScreeningEligibility):
            def __init__(self, **kwargs):
                self.thing_one = None
                super().__init__(**kwargs)

            def get_required_fields(self):
                return {"thing_one": FC(YES, "thing one must be YES")}

            def assess_eligibility(self) -> None:
                self.reasons_ineligible.update(thing_one="who me?")

        cleaned_data = dict(thing_one=YES)
        self.assertRaises(
            ScreeningEligibilityInvalidCombination,
            ScreeningEligibility,
            cleaned_data=cleaned_data,
        )

    def test_invalid_combinations2(self):
        class ScreeningEligibility(BaseScreeningEligibility):
            def __init__(self, **kwargs):
                self.thing_one = None
                super().__init__(**kwargs)

            def get_required_fields(self):
                return {"thing_one": FC(YES, "thing one must be YES")}

            def assess_eligibility(self) -> None:
                self.eligible = NO

        cleaned_data = dict(thing_one=YES)
        self.assertRaises(
            ScreeningEligibilityInvalidCombination,
            ScreeningEligibility,
            cleaned_data=cleaned_data,
        )

    def test_invalid_assess_eligibility(self):
        class ScreeningEligibility(BaseScreeningEligibility):
            def __init__(self, **kwargs):
                self.thing_one = None
                super().__init__(**kwargs)

            def get_required_fields(self):
                return {"thing_one": FC(YES, "thing one must be YES")}

            def assess_eligibility(self) -> None:
                self.eligible = None

        cleaned_data = dict(thing_one=YES)
        self.assertRaises(
            ScreeningEligibilityError,
            ScreeningEligibility,
            cleaned_data=cleaned_data,
        )



from django.test import TestCase
from edc_reportable.evaluator import ValueBoundryError

from ..age_evaluator import AgeEvaluator


class TestEvaluators(TestCase):
    def test_age(self):
        evaluator = AgeEvaluator(age_lower=18, age_lower_inclusive=True)

        self.assertRaises(ValueBoundryError, evaluator.in_bounds_or_raise, age=17)



from edc_constants.constants import FEMALE, MALE


class GenderEvaluator:

    eligible_gender = [MALE, FEMALE]

    def __init__(self, gender=None, **kwargs) -> None:  # noqa
        self.eligible = False
        self.reasons_ineligible = None
        if gender in self.eligible_gender:
            self.eligible = True
        else:
            self.reasons_ineligible = f"`{gender}` is an invalid."



from edc_constants.constants import (
    PURPOSIVELY_SELECTED,
    RANDOM_SAMPLING,
    SEQUENTIAL,
    SYSTEMATIC,
)

SELECTION_METHOD = (
    (RANDOM_SAMPLING, "Random sampling"),
    (SYSTEMATIC, "Systematically selected"),
    (SEQUENTIAL, "Sequentially selected"),
    (PURPOSIVELY_SELECTED, "Purposively selected"),
)
